const fs = require("fs-extra");
const bodyParser = require("body-parser");
const glob = require("glob");
const path = require("path");
const cors = require("cors");
const express = require("express");
const process = require("process");
/*******************************************

SEE https://gitea.ndda.fr/apxtrib/apxtrib/wiki/Devrules
To have a quick understanding and convention before doing deeply in source code

*/
// Global data  : add here globale variable that take care between RAM space anf fs access
// to make absolute path with `${__base}relativepath`
global.__base = __dirname + "/";
// app.locals.tribeids is defined later in apixtrib.js and allow express app to always have in memory a dynamic of tribeId available in req.app.locals.tribeids

// check setup
if (!fs.existsSync("/etc/nginx/nginx.conf")) {
  console.log(
    "\x1b[31m Check documentation, nginx have to be installed on this server first, no /etc/nginx/nginx.conf available, install then rerun yarn command."
  );
  process.exit();
}
if (!fs.existsSync(`${__base}nationchains/tribes/conf.json`)) {
  // this town is not set
  console.log(
    `\x1b[42m############################################################################################\x1b[0m\n\x1b[42mWellcome into apxtrib, you must first init your town and tribes by a 'yarn setup'.          \x1b[0m \n\x1b[42mThen 'yarn dev' or 'yarn startpm2' or 'yarn unittest'. Check README's project to learn more.\x1b[0m\n\x1b[42m############################################################################################\x1b[0m`
  );
  process.exit();
}
const conf = require(`${__base}nationchains/tribes/conf.json`);
// To make public this conf, careffull this localconf will be public, this the only difference between all apxtrib node
const localconf = {
  nationId: conf.nationId,
  townId: conf.townId,
  tribeId: conf.tribeId,
  comment: "Generate by apxtrib.js with minimum of information",
};
fs.outputJsonSync(
  `${__base}nationchains/www/adminapx/static/tpldata/setup_en.json`,
  localconf
);
// Run main express process

// Each tribe has a context (domain, plugins route, website ) are all describe into idx tribeId_all.json
// {"tribename":{"tribeId":"tribename","dns":[array of domain],"status":"unchain","nationId":"ants","townId":"usbfarm"}}

// dataclient .tribeids [] .DOMs [] .routes (plugins {url:name route:path}) .appname {tribeid:[website]}
//const dataclient = require( './api/models/Tribes' ).init();
const tribelist = fs.readJsonSync(
  `${__base}/nationchains/tribes/idx/tribeId_all.json`
);
let doms = conf.dns; // only dns of town during the init process
let tribeIds = [];
let routes = glob.sync(`${__base}/api/routes/*.js`).map((f) => {
  return { url: `/${path.basename(f, ".js")}`, route: f };
});
//routes={url,route} check how to add plugin tribe route later
// keep only the 2 last part (.) of domain name to validate cors with it (generic domain)
Object.keys(tribelist).forEach((t) => {
  tribelist[t].dns.forEach((d) => {
    const dm = d.split(".").slice(-2).join(".");
    if (!doms.includes(dm)) doms.push(dm);
  });
  tribeIds.push(t);
});
console.log("Allowed DOMs to access to this apxtrib server: ", doms);

const app = express();
// load express parameter from conf
Object.keys(conf.api.appset).forEach((p) => {
  app.set(p, conf.api.appset[p]);
});
// To set depending of data form or get size to send
app.use(bodyParser.urlencoded(conf.api.bodyparse.urlencoded));
// To set depending of post put json data size to send
app.use(express.json());
app.use(bodyParser.json(conf.api.bodyparse.json));
app.locals.tribeids = tribeIds;
console.log("app.locals.tribeids", app.locals.tribeids);
// Cors management
const corsOptions = {
  origin: (origin, callback) => {
    if (origin === undefined) {
      callback(null, true);
    } else if (origin.indexOf("chrome-extension") > -1) {
      callback(null, true);
    } else {
      //console.log( 'origin', origin )
      //marchais avant modif eslint const rematch = ( /^https?\:\/\/(.*)\:.*/g ).exec( origin )
      const rematch = /^https?:\/\/(.*):.*/g.exec(origin);
      //console.log( rematch )
      let tmp = origin.replace(/http.?:\/\//g, "").split(".");

      if (rematch && rematch.length > 1) tmp = rematch[1].split(".");
      //console.log( 'tmp', tmp )
      let dom = tmp[tmp.length - 1];
      if (tmp.length > 1) {
        dom = `${tmp[tmp.length - 2]}.${tmp[tmp.length - 1]}`;
      }
      console.log(
        `origin: ${origin}, dom:${dom}, CORS allowed? : ${doms.includes(dom)}`
      );
      if (doms.includes(dom)) {
        callback(null, true);
      } else {
        console.log(`Origin is not allowed by CORS`);
        callback(new Error("Not allowed by CORS"));
      }
    }
  },
  exposedHeaders: Object.keys(conf.api.exposedHeaders),
};
// CORS
app.use(cors(corsOptions));
// Static Routes // try to use nginx route instead in comments
/*app.use( express.static( `${__dirname}/nationchains/tribes/${conf.mayorId}/www/cdn/public`, {
	dotfiles: 'allow'
} ) );
*/
// Routers add any routes from /routes and /plugins
console.log("Routes available on this apxtrib instance");
console.log(routes);
routes.forEach((r) => {
  try {
    app.use(r.url, require(r.route));
  } catch (err) {
    console.log(
      `\x1b[31m!!! WARNING issue with route ${r.route} from ${r.url} check err if route is key then solve err, if not just be aware that this route won't work on your server. If you are not the maintainer and no turn around please contact the email maintainer.\x1b[0m`
    );
    console.log("raise err-:", err);
  }
});

app.listen(conf.api.port, () => {
  console.log(
    `check in your browser that api works http://${conf.dns}:${conf.api.port}`
  );
});
console.log(
  "\x1b[42m\x1b[37m",
  "Made with love for people's freedom, enjoy !!!",
  "\x1b[0m"
);
